@@ -1,14 +1,14 @@
 ;;; zk.el --- Functions for working with Zettelkasten-style linked notes -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2022-2024 Grant Rosson
+;; Copyright (C) 2022-2025 Grant Rosson
 
 ;; Author: Grant Rosson <https://github.com/localauthor>
 ;; Created: January 4, 2022
 ;; License: GPL-3.0-or-later
-;; Package-Version: 0.7
-;; Package-Revision: 599be69ae1c1
-;; Homepage: https://github.com/localauthor/zk
-;; Package-Requires: ((emacs "25.1"))
+;; Package-Version: 20250908.1240
+;; Package-Revision: 302e49432406
+;; URL: https://github.com/localauthor/zk
+;; Package-Requires: ((emacs "28.1"))
 
 ;; This program is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by the Free
@@ -70,6 +70,7 @@
 (defvar embark-multitarget-actions)
 (defvar embark-general-map)
 (defvar embark-file-map)
+(defvar embark--associated-file-fn-alist)
 
 ;;; Variables
 
@@ -303,23 +304,31 @@ Adds zk-id as an Embark target, and adds `zk-id-map' and
        (listp list)
        (null (cdr list))))
 
+(defvar zk--file-name-regexp-cache nil)
+
 (defun zk-file-name-regexp ()
   "Return the correct regexp matching zk file names.
 The regexp captures these groups:
 
 Group 1 is the zk ID.
 Group 2 is the title."
-  (concat "\\(?1:" zk-id-regexp "\\)"
-          "."
-          "\\(?2:.*?\\)"
-          "\\."
-          zk-file-extension
-          ".*"))
+  (or zk--file-name-regexp-cache
+      (setq zk--file-name-regexp-cache
+            (concat "\\(?1:" zk-id-regexp "\\)"
+                    "."
+                    "\\(?2:.*?\\)"
+                    "\\."
+                    zk-file-extension
+                    ".*"))))
+
+(defvar zk--link-regexp-cache nil)
 
 (defun zk-link-regexp ()
   "Return the correct regexp for zk links.
 The value is based on `zk-link-format' and `zk-id-regexp'."
-  (format (regexp-quote zk-link-format) zk-id-regexp))
+  (or zk--link-regexp-cache
+      (setq zk--link-regexp-cache
+            (format (regexp-quote zk-link-format) zk-id-regexp))))
 
 (defun zk--file-id (file)
   "Return the ID of the given zk FILE."
@@ -353,7 +362,7 @@ The ID is created using `zk-id-time-string-format'."
 
 (defun zk--id-list (&optional str zk-alist)
   "Return a list of zk IDs for notes in `zk-directory'.
-Optional search for STR in note title, case-insenstive. Takes an
+Optional search for STR in note title, case-insensitive. Takes an
 optional ZK-ALIST, for efficiency if `zk--id-list' is called in
 an internal loop."
   (if str
@@ -361,24 +370,23 @@ an internal loop."
             (case-fold-search t)
             (ids))
         (dolist (item zk-alist)
-          (if str
-              (when (string-match str (cadr item))
-                (push (car item) ids))
+          (when (string-match-p str (cadr item))
             (push (car item) ids)))
         ids)
-    (zk--parse-file 'id (zk--directory-files t))))
+    (let ((zk--no-gc t))
+      (ensure-list (zk--parse-file 'id (zk--directory-files t))))))
 
 (defun zk-id-p (id)
   "Return t if ID is already in use as a zk-id."
-  (when (and (listp (zk--id-list))
-             (member id (zk--id-list)))
-    t))
+  (let ((zk-id-list (zk--id-list)))
+    (when (and (listp zk-id-list)
+               (member id zk-id-list))
+      t)))
 
 (defun zk--current-id ()
   "Return the ID of zk note in current buffer."
   (or (zk--file-id buffer-file-name)
       (user-error "Not a zk file")))
-(make-obsolete 'zk--current-id 'zk--file-id "0.5")
 
 (defun zk--directory-files (&optional full regexp)
   "Return list of zk-files in `zk-directory'.
@@ -478,7 +486,7 @@ supplied. Can take a PROMPT argument."
              (display-sort-function . ,sort)
              (category . zk-file))
          (complete-with-action action files string predicate)))
-     nil t nil 'zk-file-history)))
+     nil nil nil 'zk-file-history)))
 
 (defun zk--group-function (file transform)
   "TRANSFORM completion candidate FILE to note title."
@@ -505,48 +513,49 @@ supplied. Can take a PROMPT argument."
          ,(replace-regexp-in-string zk-file-name-separator " "
                                     (match-string-no-properties 2 file))
          ,file)))
-   (zk--directory-files t)))
+   (let ((zk--no-gc t)) (zk--directory-files t))))
 
 (defun zk--parse-id (target ids &optional zk-alist)
   "Return TARGET, either `file-path or `title, from files with IDS.
 Takes a single ID, as a string, or a list of IDs. Takes an
 optional ZK-ALIST, for efficiency if `zk--parse-id' is called
 in an internal loop."
-  (cond
-   ((and (eq target 'file-path)
-         (stringp ids))
-    (car (zk--directory-files t ids)))
-   ((and (eq target 'file-path)
-         (zk--singleton-p ids))
-    (car (zk--directory-files t (car ids))))
-   (t
-    (let* ((zk-alist (or zk-alist
-                         (zk--alist)))
-           (zk-id-list (zk--id-list))
-           (return
-            (cond ((eq target 'file-path)
-                   (cond ((stringp ids)
-                          (if (member ids zk-id-list)
-                              (cddr (assoc ids zk-alist))
-                            (user-error "No file associated with %s" ids)))
-                         ((listp ids)
-                          (mapcar
-                           (lambda (x)
-                             (caddr (assoc x zk-alist)))
-                           ids))))
-                  ((eq target 'title)
-                   (cond ((stringp ids)
-                          (if (member ids zk-id-list)
-                              (cadr (assoc ids zk-alist))
-                            (user-error "No file associated with %s" ids)))
-                         ((listp ids)
-                          (mapcar
-                           (lambda (x)
-                             (cadr (assoc x zk-alist)))
-                           ids)))))))
-      (if (zk--singleton-p return)
-          (car return)
-        return)))))
+  (let ((zk--no-gc t))
+    (cond
+     ((and (eq target 'file-path)
+           (stringp ids))
+      (car (zk--directory-files t ids)))
+     ((and (eq target 'file-path)
+           (zk--singleton-p ids))
+      (car (zk--directory-files t (car ids))))
+     (t
+      (let* ((zk-alist (or zk-alist
+                           (zk--alist)))
+             (zk-id-list (zk--id-list))
+             (return
+              (cond ((eq target 'file-path)
+                     (cond ((stringp ids)
+                            (if (member ids zk-id-list)
+                                (cddr (assoc ids zk-alist))
+                              (user-error "No file associated with %s" ids)))
+                           ((listp ids)
+                            (mapcar
+                             (lambda (x)
+                               (caddr (assoc x zk-alist)))
+                             ids))))
+                    ((eq target 'title)
+                     (cond ((stringp ids)
+                            (if (member ids zk-id-list)
+                                (cadr (assoc ids zk-alist))
+                              (user-error "No file associated with %s" ids)))
+                           ((listp ids)
+                            (mapcar
+                             (lambda (x)
+                               (cadr (assoc x zk-alist)))
+                             ids)))))))
+        (if (zk--singleton-p return)
+            (car return)
+          return))))))
 
 (defun zk--parse-file (target files)
   "Return TARGET, either `id or `title, from FILES.
@@ -565,9 +574,7 @@ file extension."
                                 (match-string 2 file)))
                        (_ (signal 'wrong-type-argument
                                   `((or 'id 'title) ,target))))))
-                 (if (listp files)
-                     files
-                   (list files)))))
+                 (ensure-list files))))
     (if (zk--singleton-p result)
         (car result)
       result)))
@@ -577,7 +584,8 @@ file extension."
 (defun zk--processor (arg)
   "Return list of files.
 ARG can be zk-file or zk-id as string or list, single or multiple."
-  (let* ((zk-alist (zk--alist))
+  (let* ((zk--no-gc t)
+         (zk-alist (zk--alist))
          (files (cond
                  ((stringp arg)
                   (if (zk-file-p arg)
@@ -612,8 +620,7 @@ When NO-PROC is non-nil, bypass `zk--processor'."
 (defun zk--formatted-string (arg format)
   "Format a multi-line string from items in ARG, following FORMAT.
 ARG can be zk-file or zk-id as string or list, single or multiple."
-  (let ((items (zk--formatter arg format)))
-    (mapconcat #'identity items "\n\n")))
+  (mapconcat #'identity (zk--formatter arg format) "\n\n"))
 
 (defun zk-format-id-and-title (format id title)
   "Format ID and TITLE based on the `format-spec' FORMAT.
@@ -654,7 +661,8 @@ Adds `zk-make-link-buttons' to `find-file-hook.'"
   (interactive)
   (when (and (zk-file-p)
              zk-enable-link-buttons)
-    (let ((ids (zk--id-list)))
+    (let* ((zk--no-gc t)
+           (ids (zk--id-list)))
       (save-excursion
         (goto-char (point-min))
         (while (re-search-forward (zk-link-regexp) nil t)
@@ -688,6 +696,24 @@ Adds `zk-make-link-buttons' to `find-file-hook.'"
                                       zk-file-name-separator
                                       base-name))))
 
+(defun zk--replace-title-chars (title)
+  "Replace special characters in TITLE.
+Changes curly quotes to straight, colon to dash."
+  (with-temp-buffer
+    (insert title)
+    (format-replace-strings
+     '(("\x201C" . "\"")
+       ("\x201D" . "\"")
+       ("\x2018" . "'")
+       ("\x2019" . "'")
+       ("\n" . "")
+       (":" . "-"))
+     nil (point-min) (line-end-position))
+    (goto-char (point-min))
+    (buffer-substring-no-properties
+     (point)
+     (line-end-position))))
+
 ;;;###autoload
 (defun zk-new-note (&optional title)
   "Create a new note, insert link at point of creation.
@@ -695,9 +721,9 @@ Optional TITLE argument."
   (interactive)
   (let* ((pref-arg current-prefix-arg)
          (new-id (zk--generate-id))
-         (orig-id (ignore-errors (zk--file-id buffer-file-name)))
+         (orig-id (ignore-errors (zk--current-id)))
          (text (when (use-region-p)
-                 (buffer-substring
+                 (buffer-substring-no-properties
                   (region-beginning)
                   (region-end))))
          (title (cond (title title)
@@ -705,19 +731,20 @@ Optional TITLE argument."
                        (with-temp-buffer
                          (insert text)
                          (goto-char (point-min))
-                         (buffer-substring
+                         (buffer-substring-no-properties
                           (point)
                           (line-end-position))))
                       (t (read-string "Note title: "))))
+         (clean-title (zk--replace-title-chars title))
          (body (when (use-region-p)
                  (with-temp-buffer
                    (insert text)
                    (goto-char (point-min))
                    (forward-line 2)
-                   (buffer-substring
+                   (buffer-substring-no-properties
                     (point)
                     (point-max)))))
-         (file-name (zk--note-file-path new-id title)))
+         (file-name (zk--note-file-path new-id clean-title)))
     (unless orig-id
       (setq orig-id zk-default-backlink))
     (when (use-region-p)
@@ -733,7 +760,7 @@ Optional TITLE argument."
     (when buffer-file-name
       (save-buffer))
     (find-file file-name)
-    (funcall zk-new-note-header-function title new-id orig-id)
+    (funcall zk-new-note-header-function clean-title new-id orig-id)
     (when body (insert body))
     (when zk-enable-link-buttons (zk-make-link-buttons))
     (save-buffer)))
@@ -758,7 +785,7 @@ header title in buffer. If yes, file name changed to header
 title."
   (interactive)
   (read-only-mode -1)
-  (let* ((id (zk--file-id buffer-file-name))
+  (let* ((id (zk--current-id))
          (file-title (zk--parse-id 'title id))
          (header-title (progn
                          (save-excursion
@@ -767,16 +794,20 @@ title."
                            (buffer-substring-no-properties
                             (point)
                             (line-end-position)))))
+         (clean-file (zk--replace-title-chars file-title))
+         (clean-header (zk--replace-title-chars header-title))
          (new-title))
     (unless id
       (user-error "Not a zk file"))
-    (if (not (string= file-title header-title))
-        (if (y-or-n-p (format "Change from \"%s\" to \"%s\"? " file-title header-title))
-            (setq new-title header-title)
-          (setq new-title (read-string "New title: " file-title)))
-      (setq new-title (read-string "New title: " file-title)))
-    (when (string-match "\n" new-title)
-      (setq new-title (replace-regexp-in-string "\n" "" new-title)))
+    (setq new-title
+          (zk--replace-title-chars
+           (if (not (string= file-title header-title))
+               (if (y-or-n-p (format "Change from \"%s\" to \"%s\"? "
+                                     file-title
+                                     clean-header))
+                   clean-header
+                 (read-string "New title: " clean-file))
+             (read-string "New title: " clean-file))))
     (save-excursion
       (goto-char (point-min))
       (re-search-forward id)
@@ -792,9 +823,17 @@ title."
 
 ;;;###autoload
 (defun zk-find-file ()
-  "Find file in `zk-directory'."
+  "Find or create a note in `zk-directory'.
+Prompts for a file using `zk-select-file-function'.
+If the selected file exists, opens it.
+If it doesn't exist, creates a new note with that title."
   (interactive)
-  (find-file (funcall zk-select-file-function "Find file: ")))
+  (unless (and zk-directory (file-directory-p zk-directory))
+    (user-error "Please set `zk-directory' to a valid directory"))
+  (let ((file (funcall zk-select-file-function "Find file: ")))
+    (if (file-exists-p file)
+        (find-file file)
+      (zk-new-note file))))
 
 ;;;###autoload
 (defun zk-find-file-by-id (id)
@@ -840,21 +879,20 @@ Optionally call a custom function by setting the variable
 
 (defun zk--links-in-note-list ()
   "Return list of zk files that are linked from the current buffer."
-  (let* ((zk-alist (zk--alist))
+  (let* ((zk--no-gc t)
+         (zk-alist (zk--alist))
          (zk-ids (zk--id-list))
          id-list)
     (save-buffer)
     (save-excursion
       (goto-char (point-min))
       (while (re-search-forward (zk-link-regexp) nil t)
-        (if (member (match-string-no-properties 1) zk-ids)
-            (push (match-string-no-properties 1) id-list))))
+        (when (member (match-string-no-properties 1) zk-ids)
+          (push (match-string-no-properties 1) id-list))))
     (cond ((zk--singleton-p id-list)
            (list (zk--parse-id 'file-path id-list zk-alist)))
           (id-list
-           (zk--parse-id 'file-path (delete-dups id-list) zk-alist))
-          (t
-           (error "No zk-links in note")))))
+           (zk--parse-id 'file-path (delete-dups id-list) zk-alist)))))
 
 ;;;###autoload
 (defun zk-links-in-note ()
@@ -863,7 +901,7 @@ Optionally call a custom function by setting the variable
   (let* ((files (zk--links-in-note-list)))
     (if files
         (find-file (funcall zk-select-file-function "Links: " files))
-      (user-error "No links found"))))
+      (user-error "No zk links found"))))
 
 ;;; Insert Link
 
@@ -964,13 +1002,11 @@ ARG can be zk-file or zk-id as string or list, single or multiple."
 
 ;;;###autoload
 (defun zk-backlinks ()
-  "Select from list of all notes that link to the current note."
+  "Select from list of all notes that link to current note."
   (interactive)
-  (let* ((id (zk--file-id buffer-file-name))
-         (files (zk--backlinks-list id)))
-    (if files
-        (find-file (funcall zk-select-file-function "Backlinks: " files))
-      (user-error "No backlinks found"))))
+  (if-let ((files (zk--backlinks-list (zk--current-id))))
+      (find-file (funcall zk-select-file-function "Backlinks: " files))
+    (user-error "No backlinks found")))
 
 ;;; Search
 
@@ -1080,11 +1116,9 @@ Select TAG, with completion, from list of all tags in zk notes."
   "Find `zk-backlinks' and `zk-links-in-note' for current or selected note.
 Backlinks and Links-in-Note are grouped separately."
   (interactive)
-  (unless (zk-file-p)
-    (user-error "Not a zk file"))
-  (let* ((id (zk--file-id buffer-file-name))
-         (backlinks (ignore-errors (zk--backlinks-list id)))
-         (links-in-note (ignore-errors (zk--links-in-note-list)))
+  (let* ((id (zk--current-id))
+         (backlinks (zk--backlinks-list id))
+         (links-in-note (zk--links-in-note-list))
          (resources))
     (if (or backlinks links-in-note)
         (progn
@@ -1117,6 +1151,68 @@ Backlinks and Links-in-Note are grouped separately."
 ;;           (when (eq 'backlink (get-text-property 0 'type a))
 ;;               t))))
 
+
+;;; Modified on Date Functions
+
+(defun zk--files-modified-in-timespan (pred)
+  "Return list of files that satisfy the time PRED function."
+  (let* ((zk--no-gc t)
+         (files (zk--directory-files t))
+         (ht (make-hash-table :test #'equal :size (length files)))
+         list)
+    (dolist (x files ht)
+      (puthash x (file-attribute-modification-time
+                  (file-attributes x))
+               ht))
+    (dolist (x files list)
+      (when (funcall pred (gethash x ht))
+        (push x list)))))
+
+(defun zk--files-modified-this-week ()
+  "Return files modified in the last seven days."
+  (zk--files-modified-in-timespan
+   (lambda (mod-time)
+     (time-less-p
+      (time-subtract (current-time) (days-to-time 7))
+      mod-time))))
+
+(defun zk--files-modified-on-date (date)
+  "Return list of files modified on DATE.
+DATE must be a string in ISO 8601 date format: YYYY-MM-DD."
+  (zk--files-modified-in-timespan
+   (lambda (mod-time)
+     (equal date (format-time-string "%F" mod-time)))))
+
+(defun zk--files-modified-today ()
+  "Return files modified on today’s date."
+  (zk--files-modified-on-date
+   (format-time-string "%F" (current-time))))
+
+(defun zk--files-modified-yesterday ()
+  "Return files modified on yesterday’s date."
+  (zk--files-modified-on-date
+   (format-time-string "%F" (time-subtract
+                             (current-time)
+                             (days-to-time 1)))))
+
+(defun zk-files-modified-this-week ()
+  "Select from files modified in the last seven days."
+  (interactive)
+  (funcall zk-select-file-function "Mod’d This Week: "
+           (zk--files-modified-this-week)))
+
+(defun zk-files-modified-today ()
+  "Select from files modified on today’s date."
+  (interactive)
+  (funcall zk-select-file-function "Mod’d Today: "
+           (zk--files-modified-today)))
+
+(defun zk-files-modified-yesterday ()
+  "Select from files modified on yesterday’s date."
+  (interactive)
+  (funcall zk-select-file-function "Mod’d Yesterday: "
+           (zk--files-modified-yesterday)))
+
 (provide 'zk)
 
 ;;; zk.el ends here
